;**********************************************************************
; TITLE : Video Title Generator Rev2.1
;**********************************************************************
;	Filename	: VTG20.ASM
;    	Date Started 	: 14/8/99
;	Last edit 	: 14/8/99
;    	File Version	: 2.1
;                                                                     
;	Author		: David L. Jones - Tronnort Technology
;**********************************************************************
;                                                                     
; Files required:
;	p16F84.inc	
;                                                                     
;**********************************************************************
; Notes:            
;	USES APPROX 1MHz RC OSC.
;	For use with the Rev2.0 STV5730 design.
;	As published in Electronics Australia Sep'99
;**********************************************************************

	list      p=16F84             ; list directive to define processor
	#include <p16F84.inc>         ; processor specific variable definitions

	__CONFIG   _CP_ON & _WDT_OFF & _PWRTE_ON & _RC_OSC

;***** VARIABLE DEFINITIONS
	
;PORT PIN DEFINITIONS
#DEFINE	P_KCLK	0		; KEYBOARD CLOCK LINE
#DEFINE	P_KDATA	1		; KEYBOARD DATA LINE
#DEFINE P_CSN	4		; CSN PORT NUMBER
#DEFINE	P_CLK	5		; CLK PORT NUMBER
#DEFINE	P_DATA	6		; DATA PORT NUMBER
#DEFINE MAXCHAR	.28		; MAXIMUM NUMBER OF ROW CHARACTERS
#DEFINE SPACE	0xFF		; DUMMY CHARACTER FOR 
#DEFINE	NUMROW	.11		; MAX NUMBER OF DISPLAY ROWS-1

TEMP	EQU	0x0C		; TEMP VARIABLE
TEMP2	EQU	0x0D		; TEMP VARIABLE
CURSOR	EQU	0x0E		; SCREEN CURSOR POSITION (>0=EDIT MODE)
ROW	EQU	0x0F
BYTE2	EQU	0x10		; FIRST BYTE (MSB) OF 5730 DATA
BYTE1	EQU	0x11		; SECOND BYTE (LSB) OF 5730 DATA
KEYCODE	EQU	0x12		; RETURNED KEYBOARD CODE
BOX	EQU	0x13		; TRUE IF DISPLAY BOX IS ON
CURSORE	EQU	0x14		; TRUE IF CURSOR IS ENABLED
BLINK	EQU	0x15		; TRUE IF BLINKING IS ENABLED
VERT	EQU	0x16		; VERTICAL DISPLAY POSITION
HORIZ	EQU	0x17		; HORIZONTAL DISPLAY POSITION
HZOOM	EQU	0x18		; HORIZONTAL ZOOM
VZOOM	EQU	0x19		; VERTICAL ZOOM
ROWBIT	EQU	0x1A		; BIT FIELD TO SELECT ROW FOR HELP
TEMP3	EQU	0x1B
BOOT	EQU	0x1C		; TRUE IF BOOTED ONCE
KWAIT	EQU	0x1D		; WAIT VARIABLE FOR KEYBOARD LOOP
CHARDAT	EQU	0x1E		; START OF MEMORY TABLE FOR ROW DATA (28 BYTES)

	
;**********************************************************************
	ORG     0		; processor reset vector - start of code
	
;******************************* INITIALISATION STUFF
	CLRF	PORTB		; reset portb
	BSF	STATUS, RP0	; select bank 1
	MOVLW	B'00000011'	; PINS [RB0:1]=INPUT & ALL OTHERS=OUTPUT
 	MOVWF	TRISB		; UPDATE TRISB REGISTER TO DEFINE I/O PINS
	CLRF	TRISA
	BCF	OPTION_REG,NOT_RBPU	;ENABLE PULLUPS ON PORTB
	BCF	STATUS, RP0	; SELECT BANK 0
	BCF	INTCON, GIE	; DISABLE ALL INTERRUPTS
	CLRF	PORTB		; SET ALL PORTB PINS LOW

	CLRF	BOOT		; INITIAL BOOT
	MOVLW	B'00001000'
	MOVWF	VERT		; VERTICAL DISPLAY POSITION
	MOVLW	B'00010011'
	MOVWF	HORIZ		; HORIZONTAL DISPLAY POSITION
	CLRF	HZOOM
	CLRF	VZOOM
	CLRF	BLINK
	MOVLW	1
	MOVWF	BOX		; TURN DISPLAY BOX ON

;******************************* STV5730 CHIP INITIALISATION
INIT:
	MOVLW	1
	MOVWF	CURSOR		; CLEAR CURSOR
	MOVWF	CURSORE		; ENABLE THE CURSOR
	CLRF	ROW		; SELECT ROW 0
	CALL	SET_KCLK_HIGH	; ENABLE KEYBOARD CLOCK

	MOVLW	0x30
	MOVWF	BYTE1
	CLRF	BYTE2
	CALL	SEND16BYTE

	MOVLW	0x30
	MOVWF	BYTE1
	CLRF	BYTE2
	CALL	SEND16BYTE

	CLRF	BYTE1		;SEND RESET BYTE
	MOVLW	0xDB
	MOVWF	BYTE2
	CALL	SEND16BYTE

	MOVLW	0x10
	MOVWF	BYTE1
	CLRF	BYTE2
	CALL	SEND16BYTE
	
	CALL	ZOOM_ATTR	;SET ZOOM DATA AND WRITE POINTER
	
	MOVLW	B'00010000'	;SEND COLOR DATA
	MOVWF	BYTE1
	CLRF	BYTE2
	CALL	SEND16BYTE
	
	MOVLW	B'00010011'	;SEND CONTROL DATA
	MOVWF	BYTE1
	MOVLW	B'11101110'
	MOVWF	BYTE2
	CALL	SEND16BYTE

	MOVLW	B'00010000'	;SEND POSITION DATA
	ADDWF	VERT,0	
	MOVWF	BYTE1
	MOVLW	B'0000000'
	ADDWF	HORIZ,0
	MOVWF	BYTE2
	CALL	SEND16BYTE
	
	MOVLW	B'00010001'	;SEND MODE DATA
	MOVWF	BYTE1
	MOVLW	B'00100000'	; SET OUTPUT FOR 6dB GAIN
	MOVWF	BYTE2
	CALL	SEND16BYTE	

	CALL	ROW_ATTRIB	;UPDATE ROW ATTRIBUTES
	CALL	CLEAR_ROWS	; CLEAR ALL THE ROWS
	CLRF	ROW

	BTFSS	BOOT,0
	CALL	HELP		;DISPLAY HELP MESSAGE ONLY AT BOOT
	BSF	BOOT,0

;*****************************************************************
BEGIN:				; BEGIN MAIN LOOP
	CLRWDT			; RESET WATCHDOG TIMER

	CALL	DISPCURSOR

	CALL	GET_KEY		; GET A KEY PRESS
	CALL	GET_KEY		; GET THE KEY RELEASE CODE
	CALL	GET_KEY		; GET REPEAT KEY CODE
	CALL	SET_KCLK_HIGH

	MOVF	KEYCODE,0
	SUBLW	0x76
	BTFSC	STATUS,Z
	GOTO	INIT		;ESC KEY WAS PRESSED

	MOVF	KEYCODE,0
	SUBLW	0x66
	BTFSC	STATUS,Z
	CALL	BACKSPACE	;BACKSPACE KEY WAS PRESSED

	MOVF	KEYCODE,0
	SUBLW	0x75
	BTFSC	STATUS,Z
	CALL	UP_KEY		;UP ARROW KEY WAS PRESSED

	MOVF	KEYCODE,0
	SUBLW	0x72
	BTFSC	STATUS,Z
	CALL	DOWN_KEY	;ESC KEY WAS PRESSED

	MOVF	KEYCODE,0
	SUBLW	0x05
	BTFSC	STATUS,Z
	CALL	HELP		;F1 KEY WAS PRESSED

	MOVF	KEYCODE,0
	SUBLW	0x06
	BTFSC	STATUS,Z
	CALL	DISP_BOX	;F2 KEY WAS PRESSED

	MOVF	KEYCODE,0
	SUBLW	0x04
	BTFSC	STATUS,Z
	CALL	CURSOR_TOG	;F3 KEY WAS PRESSED

	MOVF	KEYCODE,0
	SUBLW	0x0C
	BTFSC	STATUS,Z
	CALL	CHG_BLINK	;F4 KEY WAS PRESSED

	MOVF	KEYCODE,0
	SUBLW	0x03
	BTFSC	STATUS,Z
	CALL	H_MINUS		;F5 KEY WAS PRESSED

	MOVF	KEYCODE,0
	SUBLW	0x0B
	BTFSC	STATUS,Z
	CALL	H_PLUS		;F6 KEY WAS PRESSED

	MOVF	KEYCODE,0
	SUBLW	0x83
	BTFSC	STATUS,Z
	CALL	V_MINUS		;F7 KEY WAS PRESSED

	MOVF	KEYCODE,0
	SUBLW	0x0A
	BTFSC	STATUS,Z
	CALL	V_PLUS		;F8 KEY WAS PRESSED

	MOVF	KEYCODE,0
	SUBLW	0x01
	BTFSC	STATUS,Z
	CALL	HZ_UPDATE	;F9 KEY WAS PRESSED

	MOVF	KEYCODE,0
	SUBLW	0x09
	BTFSC	STATUS,Z
	CALL	VZ_UPDATE	;F10 KEY WAS PRESSED

	MOVLW	CHARDAT
	ADDWF	CURSOR,0
	MOVWF	FSR
	MOVLW	HIGH LOOKUP
	MOVWF	PCLATH		;CORRECT FOR PAGE JUMP
	MOVF	KEYCODE,0
	CALL	LOOKUP		;CONVERT KEY CODE TO VIDEO CHARCATER
	MOVWF	KEYCODE
	SUBLW	0xFF		;IS IT A NULL CHARCATER?
	BTFSC	STATUS,Z
	GOTO	BEGIN		;NOT A DISPLAYABLE CHARACTER
	MOVF	KEYCODE,0
	MOVWF	INDF
	CALL	UPDATEROW
	CALL	DISPCURSOR
	MOVF	CURSOR,0	;CHECK FOR END OF LINE
	SUBLW	MAXCHAR
	BTFSS	STATUS,Z
	INCF	CURSOR,1	;UNDATE CURSOR

	GOTO BEGIN

; END OF MAIN LOOP
;*****************************************************************

;************************ ZOOM ATTRIB
ZOOM_ATTR:
	CLRF	BYTE1		;SELECT ZOOM REGISTER (1ST CONTROL REG)
	MOVLW	B'11001100'
	MOVWF	BYTE2
	CALL	SEND16BYTE

	MOVLW	B'00010000'	;WRITE CONTROL CODE
	MOVWF	TEMP
	BTFSC	VZOOM,1
	BSF	TEMP,3
	BTFSC	VZOOM,0
	BSF	TEMP,2
	BTFSC	HZOOM,1
	BSF	TEMP,1
	BTFSC	HZOOM,0
	BSF	TEMP,0
	MOVF	TEMP,0
	MOVWF	BYTE1

	CLRF	TEMP
	BTFSC	VZOOM,1
	BSF	TEMP,7
	BTFSC	VZOOM,0
	BSF	TEMP,6
	BTFSC	HZOOM,1
	BSF	TEMP,5
	BTFSC	HZOOM,0
	BSF	TEMP,4
	BTFSC	VZOOM,1
	BSF	TEMP,3
	BTFSC	VZOOM,0
	BSF	TEMP,2
	BTFSC	HZOOM,1
	BSF	TEMP,1
	BTFSC	HZOOM,0
	BSF	TEMP,0
	MOVF	TEMP,0
	MOVWF	BYTE2	
	CALL	SEND16BYTE
	RETURN

;************************ UPDATE POSITION ATTRIBUTE
POS_ATTRIB:
	CLRF	BYTE1		;SELECT POS REGISTER
	MOVLW	B'11001111'
	MOVWF	BYTE2
	CALL	SEND16BYTE

	MOVLW	B'00010000'	;SEND POSITION DATA
	ADDWF	VERT,0	
	MOVWF	BYTE1
	MOVLW	B'00000000'
	ADDWF	HORIZ,0
	MOVWF	BYTE2
	CALL	SEND16BYTE

	RETURN

;************************ UPDATE ROW ATTRIBUTES
ROW_ATTRIB:
	CLRF	BYTE1		;SELECT ROW 1 ATTRIBUTE
	MOVLW	B'11000000'
	MOVWF	BYTE2
	CALL	SEND16BYTE

	MOVLW	NUMROW		; NUMBER OF TIMES TO LOOP
	MOVWF	TEMP
	MOVLW	B'00010000'	; FIRST BYTE OF ROW ATTRIB
	MOVWF	BYTE1
RALOOP:				;CLEAR ALL OTHER ROWS
	MOVLW	B'10000000'
	BTFSC	BOX,0
	MOVLW	B'11000000'	;TURN CHARCATER BOX ON
	MOVWF	BYTE2
	CALL	SEND16BYTE
	DECFSZ	TEMP,1
	GOTO	RALOOP
	RETURN

;************************ HZOOM UPDATE ROUTINE
HZ_UPDATE:
	INCF	HZOOM,1		;INCREMENT
	BTFSC	HZOOM,2		;IF >4
	CLRF	HZOOM
	CALL	ZOOM_ATTR
	RETURN

;************************ VZOOM UPDATE ROUTINE
VZ_UPDATE:
	INCF	VZOOM,1		;INCREMENT
	BTFSC	VZOOM,2		;IF >4
	CLRF	VZOOM
	CALL	ZOOM_ATTR
	RETURN

;************************ HORIZONTAL DECREMENT ROUTINE
H_PLUS:
	INCF	HORIZ,1		;INCREMENT HORIZINTAL POS
	BTFSC	HORIZ,6
	DECF	HORIZ,1
	CALL	POS_ATTRIB
	RETURN

;************************ VERTICAL DECREMENT ROUTINE
V_PLUS:
	INCF	VERT,1		;INCREMENT VERTICAL POS
	BTFSC	VERT,4
	DECF	VERT,1
	CALL	POS_ATTRIB
	RETURN

;************************ HORIZONTAL DECREMENT ROUTINE
H_MINUS:
	MOVF	HORIZ,1		;SET ZERO FLAG
	BTFSS	STATUS,Z
	DECF	HORIZ,1
	CALL	POS_ATTRIB
	RETURN

;************************ VERTICAL DECREMENT ROUTINE
V_MINUS:
	MOVF	VERT,1
	BTFSS	STATUS,Z
	DECF	VERT,1
	CALL	POS_ATTRIB
	RETURN


;************************ DISPLAY BOX KEY ROUTINE
DISP_BOX:
	BTFSC	BOX,0
	GOTO	CLEARBOX
	MOVLW	1
	MOVWF	BOX
	CALL	ROW_ATTRIB
	RETURN
CLEARBOX:
	CLRF	BOX
	CALL	ROW_ATTRIB
	RETURN

;************************ CHANGE BLINK STATUS ROUTINE
CHG_BLINK:
	INCF	BLINK,1		;ENABLE BLINK
	BTFSC	BLINK,1		;IF BLINK>1?
	CLRF	BLINK		;DISABLE BLINK
	CALL	UPDATEROW
	RETURN

;************************ TOGGLE THE CURSOR ENABLE
CURSOR_TOG:
	INCF	CURSORE,1
	BTFSC	CURSORE,1	;IF BLINK>1?
	CLRF	CURSORE		;DISABLE BLINK
	CALL	DISPCURSOR
	RETURN

;************************ UP ARROW KEY ROUTINE
UP_KEY:
	MOVF	ROW,0		;GET CURRENT ROW
	BTFSC	STATUS,Z
	RETURN
	CLRF	CURSORE		;DISABLE CURSOR
	CALL	DISPCURSOR	;DISPLAY CURSOR
	BSF	CURSORE,0	;ENABLE CURSOR
	DECF	ROW,1		;DECREMENT ROW NUMBER
	MOVLW	1
	MOVWF	CURSOR		;RESET CURSOR
	CALL	DISPCURSOR
	CALL	CLR_ROW		;CLEAR THE NEW ROW DATA
	RETURN

;************************ UP ARROW KEY ROUTINE
DOWN_KEY:
	MOVF	ROW,0		;GET CURRENT ROW
	SUBLW	NUMROW-1
	BTFSC	STATUS,Z
	RETURN
	CLRF	CURSORE		;DISABLE CURSOR
	CALL	DISPCURSOR	;DISPLAY CURSOR
	BSF	CURSORE,0	;ENABLE CURSOR
	INCF	ROW,1		;DECREMENT ROW NUMBER
	MOVLW	1
	MOVWF	CURSOR		;RESET CURSOR
	CALL	DISPCURSOR
	CALL	CLR_ROW		;CLEAR THE NEW ROW DATA
	RETURN

;************************ BACKSPACE KEY ROUTINE
BACKSPACE:
	MOVF	CURSOR,1	;GET CURRENT CURSOR POS
	BTFSS	STATUS,Z
	DECF	CURSOR,1	;DECREMENT CURSOR
	MOVF	CURSOR,1
	BTFSC	STATUS,Z	;IS CURSOR AT ZERO NOW?
	INCF	CURSOR,1	;SET CURSOR BACK TO 1
	MOVLW	CHARDAT
	ADDWF	CURSOR,0	;GET THE CURSOR MEMORY POSITION
	MOVWF	FSR		;MOVE CURSOR DATA OFFSET
	MOVLW	0x0B		;SPACE CHARACTER
	MOVWF	INDF		;PUT SPACE CHARACTER AT CURSOR
	CALL	UPDATEROW
	CALL	DISPCURSOR	
	RETURN

;************************ DISPLAY CURSOR
DISPCURSOR:
	MOVF	ROW,0		; SET WRITE POINTER TO CURRENT ROW
	MOVWF	BYTE1
	CLRF	BYTE2
	CALL	SEND16BYTE

	MOVLW	B'00011111'	; STANDARD WHITE CHARCATER
	MOVWF	BYTE1		; SET BYTE1 FOR ALL SUBSEQUENT WRITES

	MOVLW	0x0B		;ASSUME SPACE CHARACTER
	BTFSC	CURSORE,0
	MOVLW	0x6C		;DISPLAY CURSOR ARROW
	MOVWF	BYTE2
	CALL	SEND16BYTE	;DISPLAY FIRST CHARCATER

	RETURN
	

;************************ UPDATE CURRENT ROW DATA TO 5730
UPDATEROW:
	MOVF	ROW,0		; SET WRITE POINTER TO CURRENT ROW
	MOVWF	BYTE1
	CLRF	BYTE2
	CALL	SEND16BYTE

	MOVLW	B'00011111'	; STANDARD WHITE CHARCATER
	MOVWF	BYTE1		; SET BYTE1 FOR ALL SUBSEQUENT WRITES

	MOVLW	CHARDAT
	MOVWF	FSR		; MOVE ROW DATA OFFSET
ULOOP1:
	MOVF	INDF,0		;GET CURRENT CHARCATER
	BTFSC	BLINK,0		;IS BLINKING ENABLED ?
	BSF	W,7		;ENABLE BLINKING BIT
	MOVWF	BYTE2
	CALL	SEND16BYTE
	INCF	FSR,1		;NEXT ROW CHAR
	MOVF	FSR,0
	SUBLW	CHARDAT+MAXCHAR
	BTFSS	STATUS,Z	; END OF ROW DATA ?
	GOTO	ULOOP1	

	RETURN

;*************	CLEAR ROW DATA
CLR_ROW:
	MOVLW	CHARDAT
	MOVWF	FSR
CRLOOP:
	MOVLW	0x0B		; SPACE CHARACTER
	MOVWF	INDF
	INCF	FSR,1		; NEXT ROW CHAR
	MOVF	FSR,0
	SUBLW	CHARDAT+MAXCHAR
	BTFSS	STATUS,Z
	GOTO	CRLOOP
	RETURN

;******************** CLEAR ALL ROWS
CLEAR_ROWS:
	CLRF	ROW
ILOOP1:				; CLEAR ALL OF THE ROWS
	CALL	CLR_ROW
	CALL	UPDATEROW
	INCF	ROW,1
	MOVF	ROW,0
	SUBLW	NUMROW
	BTFSS	STATUS,Z
	GOTO	ILOOP1
	RETURN

;***************************************** KEYBOARD INTERFACE ROUTINES
SET_KCLK_LOW:
	BCF	PORTB,P_KCLK	; SET CLK LINE LOW
	BSF	STATUS, RP0	; select bank 1
 	BCF	TRISB,P_KCLK	; SET KCLK AS AN OUTPUT
	BCF	STATUS, RP0	; SELECT BANK 0
	RETURN

SET_KCLK_HIGH:
	BSF	STATUS, RP0	; select bank 1
 	BSF	TRISB,P_KCLK	; SET KCLK AS AN INPUT
	BCF	STATUS, RP0	; SELECT BANK 0
	RETURN

WAIT_FOR_LOW:			; WAIT FOR THE KEYBOARD CLOCK TO TOGGLE
	MOVLW	0xFF
	MOVWF	KWAIT		; SET KEYBOARD TIMEOUT VARIABLE
WLOOP1:
	BTFSS	PORTB,P_KCLK	; WAIT FOR CLOCK LINE TO GO LOW
	RETURN			; CLK LINE IS LOW, RETURN
	DECFSZ	KWAIT,1		; CLOCK LINE IS NOT LOW YET, LETS UPDATE TIMEOUT
	GOTO	WLOOP1		; NOT TIMED OUT YET, KEEP WAITING

	CALL	SET_KCLK_LOW	; SET CLOCK LINE LOW TO DISABLE KBD
	GOTO	BEGIN		; TIMEOUT!, KEYBOARD ERROR OR SOMETHING
				; 	DO A HARD ABORT JUMP BACK TO START
WAIT_FOR_HIGH
WLOOP2:
	BTFSS	PORTB,P_KCLK	; IGNORE FIRST START BIT
	GOTO 	WLOOP2		; WAIT FOR CLOCK LINE TO GO HIGH
	RETURN

	;********************************************************
GET_KEY:			;****** GET A KEY FROM THE KEYBOARD
	CALL	SET_KCLK_HIGH	; ALLOW KBD TO SEND ANY DATA
	CLRF	TEMP		; TEMP TO STORE DATA BYTE RECIEVED
	MOVLW	1
	MOVWF	TEMP2		; BIT OR MASK

	CALL	WAIT_FOR_LOW	; WAIT FOR FIRST BYTE, BUT JUST IGNORE

	CALL	WAIT_FOR_HIGH
	CALL	WAIT_FOR_LOW	; GET SECOND BYTE
	BTFSC	PORTB,P_KDATA	
	BSF	TEMP,0		;SET BIT 0

	CALL	WAIT_FOR_HIGH
	CALL	WAIT_FOR_LOW	; GET THIRD BYTE
	BTFSC	PORTB,P_KDATA	
	BSF	TEMP,1		;SET BIT 1

	CALL	WAIT_FOR_HIGH
	CALL	WAIT_FOR_LOW	; GET FORTH BYTE
	BTFSC	PORTB,P_KDATA	
	BSF	TEMP,2		;SET BIT 2

	CALL	WAIT_FOR_HIGH
	CALL	WAIT_FOR_LOW	; GET FORTH BYTE
	BTFSC	PORTB,P_KDATA	
	BSF	TEMP,3		;SET BIT 3

	CALL	WAIT_FOR_HIGH
	CALL	WAIT_FOR_LOW	; GET FIFTH BYTE
	BTFSC	PORTB,P_KDATA	
	BSF	TEMP,4		;SET BIT 4

	CALL	WAIT_FOR_HIGH
	CALL	WAIT_FOR_LOW	; GET SIXTH BYTE
	BTFSC	PORTB,P_KDATA	
	BSF	TEMP,5		;SET BIT 5

	CALL	WAIT_FOR_HIGH
	CALL	WAIT_FOR_LOW	; GET SEVENTH BYTE
	BTFSC	PORTB,P_KDATA	
	BSF	TEMP,6		;SET BIT 6

	CALL	WAIT_FOR_HIGH
	CALL	WAIT_FOR_LOW	; GET EIGTH BYTE
	BTFSC	PORTB,P_KDATA	
	BSF	TEMP,7		;SET BIT 7

	CALL	WAIT_FOR_HIGH
	CALL	WAIT_FOR_LOW
	CALL	WAIT_FOR_HIGH
	CALL	WAIT_FOR_LOW
	CALL	WAIT_FOR_HIGH

	CALL	SET_KCLK_LOW	;SET CLOCK LINE LOW TO DISABLE KBD
	MOVF	TEMP,0
	MOVWF	KEYCODE

	RETURN

;***************************************** STV5730 BUS ROUTINES
SET_DATA_HIGH	MACRO		; SET DATA LINE HIGH
	BSF	PORTB,P_DATA
	ENDM

SET_DATA_LOW	MACRO			; SET DATA LINE LOW
	BCF	PORTB,P_DATA
	ENDM

SET_CSN_HIGH	MACRO			; SET CSN LINE HIGH
	BSF	PORTB,P_CSN
	ENDM

SET_CSN_LOW	MACRO			; SET CSN LINE LOW
	BCF	PORTB,P_CSN
	ENDM

SET_CLK_HIGH	MACRO			; SET CLK LINE HIGH
	BSF	PORTB,P_CLK
	ENDM

SET_CLK_LOW	MACRO			; SET CLK LINE LOW
	BCF	PORTB,P_CLK
	ENDM

;************************************
SEND16BYTE:			; SEND A 16 BIT WORD TO 5730
	CLRWDT			; CLEAR WATCHDOG
	
	;********* DATA IS PASSED IN "BYTE1" & "BYTE2"
	SET_DATA_HIGH
	SET_CLK_HIGH
	SET_CSN_HIGH
	
	SET_CSN_LOW
	SET_CLK_LOW

	MOVF	BYTE1,0
	MOVWF	TEMP3
	MOVLW	8
	MOVWF	TEMP2		;LOOP COUNTER
SENDL1:
	SET_DATA_LOW		; ASSUME DATA IS LOW
	BTFSC	TEMP3,7		; IF BIT7 IS HIGH THEN DO
	SET_DATA_HIGH
	SET_CLK_HIGH
	SET_CLK_LOW
	RLF	TEMP3,1		; SHIFT THE BITS LEFT BY ONE
	DECFSZ	TEMP2,1
	GOTO	SENDL1

	MOVF	BYTE2,0
	MOVWF	TEMP3
	MOVLW	7
	MOVWF	TEMP2		; LOOP COUNTER
SENDL2:
	SET_DATA_LOW		; ASSUME DATA IS LOW
	BTFSC	TEMP3,7		; IF BIT7 IS HIGH THEN DO
	SET_DATA_HIGH
	SET_CLK_HIGH
	SET_CLK_LOW
	RLF	TEMP3,1		; SHIFT THE BITS LEFT BY ONE
	DECFSZ	TEMP2,1
	GOTO	SENDL2

	SET_DATA_LOW		; ASSUME DATA IS LOW
	BTFSC	TEMP3,7		; IF BIT0 IS HIGH THEN DO
	SET_DATA_HIGH
	SET_CLK_HIGH

	SET_CSN_HIGH		; END OF DATA

	RETURN

;************************************************************

;************************ HELP
HELP:
	CALL	CLEAR_ROWS
	CLRF	HZOOM
	CLRF	VZOOM
	CALL	ZOOM_ATTR
	CLRF	ROW
	MOVLW	B'00000001'
	MOVWF	ROWBIT
HELPL1:
	CALL	GET_ROW
	CALL	UPDATEROW	
	BCF	STATUS,C
	RLF	ROWBIT,1
	INCF	ROW,1
	MOVF	ROW,0
	SUBLW	6
	BTFSS	STATUS,Z
	GOTO	HELPL1
	RETURN

;*************	GET HELP ROW DATA
GET_ROW:
	MOVLW	CHARDAT
	MOVWF	FSR
GRLOOP:
	MOVLW	HIGH HROW1
	MOVWF	PCLATH		;CORRECT FOR PAGE JUMP
	MOVLW	CHARDAT
	SUBWF	FSR,0		;GET CHARACTER NUMBER
	BTFSC	ROWBIT,0
	CALL	HROW1
	BTFSC	ROWBIT,1
	CALL	HROW2
	BTFSC	ROWBIT,2
	CALL	HROW3
	BTFSC	ROWBIT,3
	CALL	HROW4
	BTFSC	ROWBIT,4
	CALL	HROW5
	BTFSC	ROWBIT,5
	CALL	HROW6

	MOVWF	INDF
	INCF	FSR,1		; NEXT ROW CHAR
	MOVF	FSR,0
	SUBLW	CHARDAT+MAXCHAR
	BTFSS	STATUS,Z
	GOTO	GRLOOP
	RETURN

;*********************************************************
	ORG	0x210
HROW1:
	ADDWF	PCL,1
	RETLW	0x0B	;
	RETLW	0x0B	;
	RETLW	0x0B	;
	RETLW	0x0B	;
	RETLW	0x0B	;
	RETLW	0x21	;V
	RETLW	0x1F	;T
	RETLW	0x12	;G
	RETLW	0x0A	;-
	RETLW	0x00	;0
	RETLW	0x01	;1
	RETLW	0x0B	;
	RETLW	0x21	;V
	RETLW	0x2E	;e
	RETLW	0x3B	;r
	RETLW	0x3C	;s
	RETLW	0x32	;i
	RETLW	0x38	;o
	RETLW	0x37	;n
	RETLW	0x0B	;
	RETLW	0x02	;2
	RETLW	0x27	;.
	RETLW	0x01	;1
	RETLW	0x0B	;
	RETLW	0x0B	;
	RETLW	0x0B	;
	RETLW	0x0B	;
	RETLW	0x0B	;

HROW2:
	ADDWF	PCL,1
	RETLW	0x0E	;C
	RETLW	0x38	;o
	RETLW	0x39	;p
	RETLW	0x42	;y
	RETLW	0x3B	;r
	RETLW	0x32	;i
	RETLW	0x30	;g
	RETLW	0x31	;h
	RETLW	0x3D	;t
	RETLW	0x29	;'
	RETLW	0x09	;9
	RETLW	0x09	;9
	RETLW	0x0B	;
	RETLW	0x1F	;T
	RETLW	0x1D	;R
	RETLW	0x1A	;O
	RETLW	0x19	;N
	RETLW	0x19	;N
	RETLW	0x1A	;O
	RETLW	0x1D	;R
	RETLW	0x1F	;T
	RETLW	0x0B	;
	RETLW	0x1F	;T
	RETLW	0x2E	;e
	RETLW	0x2C	;c
	RETLW	0x31	;h
	RETLW	0x27	;.
	RETLW	0x0B	;

HROW3:
	ADDWF	PCL,1
	RETLW	0x11	;F
	RETLW	0x01	;1
	RETLW	0x78	;=
	RETLW	0x13	;H
	RETLW	0x10	;E
	RETLW	0x17	;L
	RETLW	0x1B	;P
	RETLW	0x0B	;
	RETLW	0x0B	;
	RETLW	0x11	;F
	RETLW	0x02	;2
	RETLW	0x78	;=
	RETLW	0x0D	;B
	RETLW	0x1A	;O
	RETLW	0x1D	;R
	RETLW	0x0F	;D
	RETLW	0x10	;E
	RETLW	0x1D	;R
	RETLW	0x0B	;
	RETLW	0x11	;F
	RETLW	0x03	;3
	RETLW	0x78	;=
	RETLW	0x0E	;C
	RETLW	0x20	;U
	RETLW	0x1D	;R
	RETLW	0x1E	;S
	RETLW	0x1A	;O
	RETLW	0x1D	;R

HROW4:
	ADDWF	PCL,1
	RETLW	0x11	;F
	RETLW	0x04	;4
	RETLW	0x78	;=
	RETLW	0x0D	;B
	RETLW	0x17	;L
	RETLW	0x14	;I
	RETLW	0x19	;N
	RETLW	0x16	;K
	RETLW	0x0B	;
	RETLW	0x11	;F
	RETLW	0x05	;5
	RETLW	0x78	;=
	RETLW	0x13	;H
	RETLW	0x1A	;O
	RETLW	0x1D	;R
	RETLW	0x14	;I
	RETLW	0x25	;Z
	RETLW	0x0A	;-
	RETLW	0x0B	;
	RETLW	0x11	;F
	RETLW	0x06	;6
	RETLW	0x78	;=
	RETLW	0x13	;H
	RETLW	0x1A	;O
	RETLW	0x1D	;R
	RETLW	0x14	;I
	RETLW	0x25	;Z
	RETLW	0x5E	;+

HROW5:
	ADDWF	PCL,1
	RETLW	0x11	;F
	RETLW	0x07	;7
	RETLW	0x78	;=
	RETLW	0x21	;V
	RETLW	0x10	;E
	RETLW	0x1D	;R
	RETLW	0x1F	;T
	RETLW	0x0A	;-
	RETLW	0x0B	;
	RETLW	0x11	;F
	RETLW	0x08	;8
	RETLW	0x78	;=
	RETLW	0x21	;V
	RETLW	0x10	;E
	RETLW	0x1D	;R
	RETLW	0x1F	;T
	RETLW	0x5E	;+
	RETLW	0x0B	;
	RETLW	0x0B	;
	RETLW	0x11	;F
	RETLW	0x09	;9
	RETLW	0x78	;=
	RETLW	0x13	;H
	RETLW	0x25	;Z
	RETLW	0x1A	;O
	RETLW	0x1A	;O
	RETLW	0x18	;M
	RETLW	0x0B	;

HROW6:
	ADDWF	PCL,1
	RETLW	0x11	;F
	RETLW	0x01	;1
	RETLW	0x00	;0
	RETLW	0x78	;=
	RETLW	0x21	;V
	RETLW	0x25	;Z
	RETLW	0x1A	;O
	RETLW	0x1A	;O
	RETLW	0x18	;M
	RETLW	0x0B	;
	RETLW	0x0B	;
	RETLW	0x0B	;
	RETLW	0x0B	;
	RETLW	0x0B	;
	RETLW	0x0B	;
	RETLW	0x0B	;
	RETLW	0x0B	;
	RETLW	0x0B	;
	RETLW	0x0B	;
	RETLW	0x10	;E
	RETLW	0x1E	;S
	RETLW	0x0E	;C
	RETLW	0x78	;=
	RETLW	0x0E	;C
	RETLW	0x17	;L
	RETLW	0x10	;E
	RETLW	0x0C	;A
	RETLW	0x1D	;R


;************************************************* LOOKUP TABLE
; CONVERTS KEYBOARD CODES INTO stv5730 DISPLAY CODES
	ORG	0x0300
LOOKUP:
	ADDWF	PCL,1
	RETLW	SPACE	;0
	RETLW	SPACE	;1
	RETLW	SPACE	;2
	RETLW	SPACE	;3
	RETLW	SPACE	;4
	RETLW	SPACE	;5
	RETLW	SPACE	;6
	RETLW	SPACE	;7
	RETLW	SPACE	;8
	RETLW	SPACE	;9
	RETLW	SPACE	;A
	RETLW	SPACE	;B
	RETLW	SPACE	;C
	RETLW	0x71	;D
	RETLW	SPACE	;E
	RETLW	SPACE	;F
	RETLW	SPACE	;10
	RETLW	SPACE	;11
	RETLW	SPACE	;12
	RETLW	SPACE	;13
	RETLW	SPACE	;14
	RETLW	0x1C	;15
	RETLW	0x01	;16
	RETLW	SPACE	;17
	RETLW	SPACE	;18
	RETLW	SPACE	;19
	RETLW	0x25	;1A
	RETLW	0x1E	;1B
	RETLW	0x0C	;1C
	RETLW	0x22	;1D
	RETLW	0x02	;1E
	RETLW	SPACE	;1F
	RETLW	SPACE	;20
	RETLW	0x0E	;21
	RETLW	0x23	;22
	RETLW	0x0F	;23
	RETLW	0x10	;24
	RETLW	0x04	;25
	RETLW	0x03	;26
	RETLW	SPACE	;27
	RETLW	SPACE	;28
	RETLW	0x0B	;29
	RETLW	0x21	;2A
	RETLW	0x11	;2B
	RETLW	0x1F	;2C
	RETLW	0x1D	;2D
	RETLW	0x05	;2E
	RETLW	SPACE	;2F
	RETLW	SPACE	;30
	RETLW	0x19	;31
	RETLW	0x0D	;32
	RETLW	0x13	;33
	RETLW	0x12	;34
	RETLW	0x24	;35
	RETLW	0x06	;36
	RETLW	SPACE	;37
	RETLW	SPACE	;38
	RETLW	SPACE	;39
	RETLW	0x18	;3A
	RETLW	0x15	;3B
	RETLW	0x20	;3C
	RETLW	0x07	;3D
	RETLW	0x08	;3E
	RETLW	SPACE	;3F
	RETLW	SPACE	;40
	RETLW	SPACE	;41
	RETLW	0x16	;42
	RETLW	0x14	;43
	RETLW	0x1A	;44
	RETLW	0x00	;45
	RETLW	0x09	;46
	RETLW	SPACE	;47
	RETLW	SPACE	;48
	RETLW	0x27	;49
	RETLW	0x28	;4A
	RETLW	0x17	;4B
	RETLW	0x26	;4C
	RETLW	0x1B	;4D
	RETLW	0x0A	;4E
	RETLW	SPACE	;4F
	RETLW	SPACE	;50
	RETLW	SPACE	;51
	RETLW	0x29	;52
	RETLW	SPACE	;53
	RETLW	SPACE	;54
	RETLW	0x78	;55
	RETLW	SPACE	;56
	RETLW	SPACE	;57
	RETLW	SPACE	;58
	RETLW	SPACE	;59
	RETLW	SPACE	;5A
	RETLW	SPACE	;5B
	RETLW	SPACE	;5C
	RETLW	SPACE	;5D
	RETLW	SPACE	;5E
	RETLW	SPACE	;5F
	RETLW	SPACE	;60
	RETLW	SPACE	;61
	RETLW	SPACE	;62
	RETLW	SPACE	;63
	RETLW	SPACE	;64
	RETLW	SPACE	;65
	RETLW	SPACE	;66
	RETLW	SPACE	;67
	RETLW	SPACE	;68
	RETLW	0x7C	;69
	RETLW	SPACE	;6A
	RETLW	SPACE	;6B
	RETLW	0x7B	;6C
	RETLW	SPACE	;6D
	RETLW	SPACE	;6E
	RETLW	SPACE	;6F
	RETLW	SPACE	;70
	RETLW	0x27	;71
	RETLW	SPACE	;72
	RETLW	SPACE	;73
	RETLW	SPACE	;74
	RETLW	SPACE	;75
	RETLW	SPACE	;76
	RETLW	SPACE	;77
	RETLW	SPACE	;78
	RETLW	0x5E	;79
	RETLW	0x7D	;7A
	RETLW	0x0A	;7B
	RETLW	0x5F	;7C
	RETLW	0x7A	;7D
	RETLW	SPACE	;7E
	RETLW	SPACE	;7F
	RETLW	SPACE	;80
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE
	RETLW	SPACE

	END			; END OF PROGRAM
